import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zi_dct0",
    version="0.0.6",
    author="Kien Nguyen",
    author_email="kiennt23@gmail.com",
    description="ZI DCT0 implementation in python",
    long_description=long_description,
    url="https://github.com/kiennt23/zi_dct0.git",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)



import logging
from enum import Enum


class DCEventType(Enum):
    UPTURN = 1
    DOWNTURN = 2
    OVERSHOOT = 3


class TradeStrategy(Enum):
    TF = 1
    CT = 2


class Config:
    def __init__(self, strategy, delta_p, initial_mode=None, initial_p_ext=None):
        self.strategy = strategy
        self.delta_p = delta_p
        if initial_mode is None and initial_p_ext is None:
            if strategy == TradeStrategy.TF:
                self.initial_mode = DCEventType.UPTURN
                self.initial_p_ext = 0.0
            else:
                self.initial_mode = DCEventType.DOWNTURN
                self.initial_p_ext = 1000000000.0
        else:
            self.initial_mode = initial_mode
            self.initial_p_ext = initial_p_ext


class ZI_DCT0:
    def __init__(self, logger, config: Config):
        self.logger = logger
        self.config = config
        self.mode = config.initial_mode
        self.current_event = DCEventType.OVERSHOOT
        self.p_ext = config.initial_p_ext
        pass

    def observe(self, p_t):
        if self.config is None:
            self.logger.error('No configuration')
            return
        self.logger.debug('p_ext={} p_t={}'.format(self.p_ext, p_t))
        if self.mode == DCEventType.UPTURN:
            if p_t <= self.p_ext * (1.0 - self.config.delta_p):
                self.mode = DCEventType.DOWNTURN
                self.current_event = DCEventType.DOWNTURN
                self.p_ext = p_t
            else:
                self.p_ext = max([self.p_ext, p_t])
                self.current_event = DCEventType.OVERSHOOT

        else:  # initial_mode is DOWNTURN
            if p_t >= self.p_ext * (1.0 + self.config.delta_p):
                self.mode = DCEventType.UPTURN
                self.current_event = DCEventType.UPTURN
                self.p_ext = p_t
            else:
                self.p_ext = min([self.p_ext, p_t])
                self.current_event = DCEventType.OVERSHOOT

        return self.current_event

    def is_buy_signaled(self):
        buy_signaled = (TradeStrategy.TF == self.config.strategy and DCEventType.UPTURN == self.current_event) or (
            TradeStrategy.CT == self.config.strategy and DCEventType.DOWNTURN == self.current_event)
        return buy_signaled

    def is_sell_signaled(self):
        sell_signaled = (TradeStrategy.TF == self.config.strategy and DCEventType.DOWNTURN == self.current_event) or (
                TradeStrategy.CT == self.config.strategy and DCEventType.UPTURN == self.current_event)
        return sell_signaled



import logging
import unittest
from core.algo import *


class ZI_DCT0Test(unittest.TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.DEBUG)
        pass

    def tearDown(self):
        pass

    def test_should_return_overshoot(self):
        config = Config(TradeStrategy.TF, 0.01)
        dct0_runner = ZI_DCT0(self.logger, config)
        observe = dct0_runner.observe(100)
        self.assertEqual(DCEventType.UPTURN, dct0_runner.mode)
        self.assertEqual(100, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        pass

    def test_should_turn_downturn(self):
        config = Config(TradeStrategy.TF, 0.01)
        dct0_runner = ZI_DCT0(self.logger, config)
        price_series = [100, 99.9, 100.1, 99.5, 100.5, 99]
        observe = dct0_runner.observe(price_series[0])
        self.assertEqual(DCEventType.UPTURN, dct0_runner.mode)
        self.assertEqual(100, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[1])
        self.assertEqual(DCEventType.UPTURN, dct0_runner.mode)
        self.assertEqual(100, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[2])
        self.assertEqual(DCEventType.UPTURN, dct0_runner.mode)
        self.assertEqual(100.1, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[3])
        self.assertEqual(DCEventType.UPTURN, dct0_runner.mode)
        self.assertEqual(100.1, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[4])
        self.assertEqual(DCEventType.UPTURN, dct0_runner.mode)
        self.assertEqual(100.5, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[5])
        self.assertEqual(DCEventType.DOWNTURN, dct0_runner.mode)
        self.assertEqual(99, dct0_runner.p_ext)
        self.assertEqual(DCEventType.DOWNTURN, observe)
        pass

    def test_TF_turn_downturn_should_sell(self):
        config = Config(TradeStrategy.TF, 0.01)
        dct0_runner = ZI_DCT0(self.logger, config)
        price_series = [100, 99.9, 100.1, 99.5, 100.5, 99]
        observe = dct0_runner.observe(price_series[0])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[1])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[2])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[3])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[4])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[5])
        self.assertTrue(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        self.assertEqual(DCEventType.DOWNTURN, dct0_runner.mode)
        self.assertEqual(99, dct0_runner.p_ext)
        self.assertEqual(DCEventType.DOWNTURN, observe)
        pass

    def test_should_turn_upturn(self):
        config = Config(TradeStrategy.TF, 0.01, DCEventType.DOWNTURN, 1000000000.0)
        dct0_runner = ZI_DCT0(self.logger, config)
        price_series = [101, 100.9, 101.1, 100.5, 101.5, 100, 101]
        observe = dct0_runner.observe(price_series[0])
        self.assertEqual(DCEventType.DOWNTURN, dct0_runner.mode)
        self.assertEqual(101, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[1])
        self.assertEqual(DCEventType.DOWNTURN, dct0_runner.mode)
        self.assertEqual(100.9, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[2])
        self.assertEqual(DCEventType.DOWNTURN, dct0_runner.mode)
        self.assertEqual(100.9, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[3])
        self.assertEqual(DCEventType.DOWNTURN, dct0_runner.mode)
        self.assertEqual(100.5, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[4])
        self.assertEqual(DCEventType.DOWNTURN, dct0_runner.mode)
        self.assertEqual(100.5, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[5])
        self.assertEqual(DCEventType.DOWNTURN, dct0_runner.mode)
        self.assertEqual(100, dct0_runner.p_ext)
        self.assertEqual(DCEventType.OVERSHOOT, observe)
        observe = dct0_runner.observe(price_series[6])
        self.assertEqual(DCEventType.UPTURN, dct0_runner.mode)
        self.assertEqual(101, dct0_runner.p_ext)
        self.assertEqual(DCEventType.UPTURN, observe)
        pass

    def test_TF_turn_upturn_should_buy(self):
        config = Config(TradeStrategy.TF, 0.01, DCEventType.DOWNTURN, 1000000000.0)
        dct0_runner = ZI_DCT0(self.logger, config)
        price_series = [101, 100.9, 101.1, 100.5, 101.5, 100, 101]
        observe = dct0_runner.observe(price_series[0])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[1])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[2])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[3])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[4])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[5])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertFalse(dct0_runner.is_buy_signaled())
        observe = dct0_runner.observe(price_series[6])
        self.assertFalse(dct0_runner.is_sell_signaled())
        self.assertTrue(dct0_runner.is_buy_signaled())
        self.assertEqual(DCEventType.UPTURN, dct0_runner.mode)
        self.assertEqual(101, dct0_runner.p_ext)
        self.assertEqual(DCEventType.UPTURN, observe)
        pass
